'''Package initialisation for asq.

No submodules are imported.  Names from submodules should be imported directly
from that submodule.
'''
from .version import __version__

__author__ = 'Robert Smallshire'
